unit ExtrasUnit;

interface
  uses graphics, SysUtils;

  function LwrChr (Key : Word) : Char;
  procedure NumOnly (var Key : Char);
  function IsJPEG (fName : TFileName) : boolean;
  function BMPorJPEG (fName : TFileName) : boolean;
  function strcmp (s1 : string; s2 : string) : boolean;
  function GetParam : string;
  function PixelFormatToInt (pf : TPixelFormat) : integer;
  function IntToPixelFormat (i : integer) : TPixelFormat;
  function AddExt (fName : TFileName; ext : string) : string;
  function RemoveExt (fName : TFileName) : TFileName;

implementation

function LwrChr (Key : Word) : Char;
begin
  result := Chr (Key);

  if result in ['A'..'Z'] then
    result := Chr (Key + Ord ('a') - Ord ('A'));
end;

procedure NumOnly (var Key : Char);
begin
  if not (Key in ['0'..'9']) and (Key <> #8) then
    Key := #0;
end;

function IsJPEG (fName : TFileName) : boolean;
begin
  result := strcmp (ExtractFileExt (fName), '.jpg');
end;

function BMPorJPEG (fName : TFileName) : boolean;
begin
  result := strcmp (ExtractFileExt (fName), '.bmp')
    or strcmp (ExtractFileExt (fName), '.jpg');
end;

function strcmp (s1 : string; s2 : string) : boolean;
begin
  result := StrIComp (PChar (s1), PChar (s2)) = 0;
end;

function GetParam : string;
var i : byte;
begin
  if ParamCount > 0 then
    for i := 1 to ParamCount do begin
      result := result + ParamStr (i);

      if i < ParamCount then
        result := result + #32;
    end;
end;


function PixelFormatToInt (pf : TPixelFormat) : integer;
begin
  case pf of
    pf1bit : result := 0;
    pf4bit : result := 1;
    pf8bit : result := 2;
    pf15bit : result := 3;
    pf16bit : result := 4;
    pf24bit : result := 5;
    pf32bit : result := 6;
    pfCustom : result := 7;
    else result := 8;
  end;
end;

function IntToPixelFormat (i : integer) : TPixelFormat;
begin
  case i of
    0 : result := pf1bit;
    1 : result := pf4bit;
    2 : result := pf8bit;
    3 : result := pf15bit;
    4 : result := pf16bit;
    5 : result := pf24bit;
    6 : result := pf32bit;
    else result := pfCustom;
  end;
end;

function AddExt (fName : TFileName; ext : string) : string;
begin
  if strcmp (ext, ExtractFileExt (fname)) then exit;

  if ext <> '' then
    if ext[1] <> '.' then
      ext := '.' + ext;

  if fName[length (fName)] = '.' then
    delete (fName, length (fName), 1);

  result := fName + ext;
end;

function RemoveExt (fName : TFileName) : TFileName;
var i : word;
begin
  i := pos ('.', fName);
  if i = 0 then
    result := fName
  else
    result := copy (fName, 1, i - 1);
end;

end.

