unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtDlgs, ExtCtrls, StdCtrls, jpeg, Printers;

const
  ProgName = 'BMP & JPEG Converter';

type
  TMainForm = class(TForm)
    OpenDlg: TOpenPictureDialog;
    SaveDlg: TSavePictureDialog;
    MainMenu1: TMainMenu;
    FileMenu: TMenuItem;
    OpenItem: TMenuItem;
    SaveAsItem: TMenuItem;
    N1: TMenuItem;
    ExitItem: TMenuItem;
    ImageMnu: TMenuItem;
    PropertiesItem: TMenuItem;
    FlipItem: TMenuItem;
    PopupMenu1: TPopupMenu;
    OpenPopItem: TMenuItem;
    SaveAsPopItem: TMenuItem;
    PrintPopItem: TMenuItem;
    PropertiesPopItem: TMenuItem;
    Image1: TImage;
    Shape1: TShape;
    ReloadItem: TMenuItem;
    PrintItem: TMenuItem;
    N3: TMenuItem;
    SetupItem: TMenuItem;
    PrintDialog1: TPrintDialog;
    PrinterSetupDialog1: TPrinterSetupDialog;
    AboutItem: TMenuItem;
    FlipPopItem: TMenuItem;
    procedure ExitItemClick(Sender: TObject);
    procedure OpenItemClick(Sender: TObject);
    procedure SaveAsItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PropertiesItemClick(Sender: TObject);
    procedure FlipItemClick(Sender: TObject);
    procedure ReloadItemClick(Sender: TObject);
    procedure SetupItemClick(Sender: TObject);
    procedure PrintItemClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure AboutItemClick(Sender: TObject);
  private
    { Private declarations }
    FileName : TFileName;
    function CanOpen (var fName : TFileName) : boolean;
    function OpenParam : boolean;
    function OpenFile (fName : TFileName) : boolean;
    procedure DoMenus;
  public
    BitMap : TBitMap;
    CompQual : TJPEGQualityRange;
    procedure Rotate90;
    procedure Rotate270;
    procedure FlipHorz;
    procedure FlipVert;
    procedure PaintImage;
    procedure AssignFileName (fName : TFileName);
    function GetFileName : TFileName;
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses PropertiesUnit, FlipUnit, ExtrasUnit, AboutUnit;

{$R *.DFM}

procedure TMainForm.Rotate90;
var
  x, y : word;
  BitMap2 : TBitMap;
begin
  BitMap2 := TBitMap.Create;
  with BitMap do begin
    BitMap2.Width := Height;
    BitMap2.Height := Width;

    for y := 0 to Height - 1 do
      for x := 0 to Width - 1 do
        BitMap2.Canvas.Pixels[y, Width - x - 1] := Canvas.Pixels[x, y];

    Assign (BitMap2);
  end;//with...
end;

procedure TMainForm.Rotate270;
var
  x, y : word;
  BitMap2 : TBitMap;
begin
  BitMap2 := TBitMap.Create;
  with BitMap do begin
    BitMap2.Width := Height;
    BitMap2.Height := Width;

    for y := 0 to Height - 1 do
      for x := 0 to Width - 1 do
        BitMap2.Canvas.Pixels[Height - y - 1, x] := Canvas.Pixels[x, y];

    Assign (BitMap2);
  end;//with...
end;

procedure TMainForm.FlipHorz;
var
  x, y : word;
  HColor : TColor;
begin
  with BitMap do
    for y := 0 to Height - 1 do
      for x := 0 to Width div 2 do begin
        HColor := Canvas.Pixels[x, y];
        Canvas.Pixels[x, y] := Canvas.Pixels[Width - x - 1, y];
        Canvas.Pixels[Width - x - 1, y ] := HColor;
      end;
end;

procedure TMainForm.FlipVert;
var
  x, y : word;
  HColor : TColor;
begin
  with BitMap do
    for y := 0 to Height div 2 do
      for x := 0 to Width - 1 do begin
        HColor := Canvas.Pixels[x, y];
        Canvas.Pixels[x, y] := Canvas.Pixels[x, Height - y - 1];
        Canvas.Pixels[x, Height - y - 1] := HColor;
      end;
end;

function TMainForm.CanOpen (var fName : TFileName) : boolean;
var f : TFileName;
begin
  if not BMPorJPEG (fName) then begin
    f := AddExt (fName, 'bmp');
    if FileExists (f) then fName := f;

    f := AddExt (fName, 'jpg');
    if FileExists (f) then fName := f;
  end;

  if not FileExists (fName) then begin
    ShowMessage ('File "' + fName + '" does not exist');
    result := false;
    exit;
  end;

  if not BMPorJPEG (fName) then begin
    ShowMessage ('File "' + fName + '" is not a bmp or jpg file');
    result := false;
    exit;
  end;

  result := true;
end;

function TMainForm.OpenParam;
var s : string;
begin
  s := GetParam;
  if s = '' then begin
    result := false;
    exit;
  end;

  result := OpenFile (s);
end;

procedure TMainForm.DoMenus;
var b : boolean;
begin
  b := FileName <> '';
  ReloadItem.Enabled := b;
  SetupItem.Enabled := b;
  PrintItem.Enabled := b;
  PrintPopItem.Enabled := b;
  SaveAsItem.Enabled := b;
  SaveAsPopItem.Enabled := b;
  PropertiesItem.Enabled := b;
  PropertiesPopItem.Enabled := b;
  FlipItem.Enabled := b;
  FlipPopItem.Enabled := b;
end;

function TMainForm.OpenFile (fName : TFileName) : boolean;
var JPEGImage : TJPEGImage;
begin
  if not CanOpen (fName) then begin
    result := false;
    exit;
  end;

  if IsJPEG (fName) then begin
    JPEGImage := TJPEGImage.Create;
    JPEGImage.LoadFromFile (fName);
    CompQual := JPEGImage.CompressionQuality;
    BitMap.Assign (JPEGImage);
  end else begin
    BitMap.LoadFromFile (fName);
    CompQual := 80;
  end;

  PaintImage;
  AssignFileName (fName);
  result := true;
end;

procedure TMainForm.PaintImage;
begin
  if (BitMap.Height < Shape1.Height)
  and (BitMap.Width < Shape1.Width) then begin
    Image1.Width := BitMap.Width;
    Image1.Height := BitMap.Height;
  end else
    if BitMap.Height > BitMap.Width then begin
      Image1.Height := Shape1.Height - 2;
      Image1.Width := BitMap.Width * Shape1.Width div BitMap.Height - 2;
    end else begin
      Image1.Height := BitMap.Height * Shape1.Height div BitMap.Width - 2;
      Image1.Width := Shape1.Width - 2;
    end;

  Image1.Top := (Shape1.Height - Image1.Height) div 2;
  Image1.Left := (Shape1.Width - Image1.Width) div 2;
  Image1.Picture.Bitmap.Assign (BitMap);
end;

procedure TMainForm.AssignFileName (fName : TFileName);
begin
  FileName := fName;

  Caption := ProgName;
  if FileName <> '' then
    Caption := Caption + ' - ' + FileName;
      //ExtractFileName (FileName);

  Application.Title := Caption;
  DoMenus;
end;

function TMainForm.GetFileName : TFileName;
begin
  result := FileName;
end;

//************Class Functions******************

procedure TMainForm.ExitItemClick(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TMainForm.OpenItemClick(Sender: TObject);
begin
  if not OpenDlg.Execute then exit;
  OpenFile (OpenDlg.FileName);
end;

procedure TMainForm.SaveAsItemClick(Sender: TObject);
var JPEGImage : TJPEGImage;
begin
  SaveDlg.FilterIndex := Ord (IsJPEG (FileName)) + 1;
  SaveDlg.FileName := RemoveExt (FileName);

  if not SaveDlg.Execute then exit;

  if not BMPorJPEG (SaveDlg.FileName) then
    if SaveDlg.FilterIndex = 1 then
      SaveDlg.FileName := AddExt (SaveDlg.FileName, 'jpg')
    else
      SaveDlg.FileName := AddExt (SaveDlg.FileName, 'bmp');

  if FileExists (SaveDlg.FileName) then
    if MessageDlg ('"' + SaveDlg.FileName + '" exists. Overwrite?',
                   mtConfirmation, [mbYes, mbNo], 0) = mrNo then exit;

  AssignFileName (SaveDlg.FileName);

  if IsJPEG (FileName) then begin
    JPEGImage := TJPEGImage.Create;
    JPEGImage.CompressionQuality := CompQual;
    JPEGImage.Assign (BitMap);
    JPEGImage.SaveToFile (FileName);
  end else
    BitMap.SaveToFile (FileName);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.HintHidePause := 3 * Application.HintHidePause;
  BitMap := TBitMap.Create;
  if not OpenParam then AssignFileName ('');
end;

procedure TMainForm.PropertiesItemClick(Sender: TObject);
begin
  PropertiesForm.ShowModal;
end;

procedure TMainForm.FlipItemClick(Sender: TObject);
begin
  FlipForm.ShowModal;
end;

procedure TMainForm.ReloadItemClick(Sender: TObject);
begin
  if MessageDlg ('Sure you want to reload ' + FileName + '?',
                 mtConfirmation, [mbYes, mbNo], 0) = mrNo then exit;

  OpenFile (FileName);
end;

procedure TMainForm.SetupItemClick(Sender: TObject);
begin
  PrinterSetupDialog1.Execute;
end;

procedure TMainForm.PrintItemClick(Sender: TObject);
begin
  if not PrintDialog1.Execute then exit;
  Printer.BeginDoc;
  Printer.Canvas.Draw (0, 0, BitMap);
  Printer.EndDoc;
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  PaintImage;
end;

procedure TMainForm.AboutItemClick(Sender: TObject);
begin
  AboutForm.ShowModal;
end;

end.

